import React, { useState, useEffect, useCallback, useRef } from 'react';
import { useParams, useNavigate } from 'react-router-dom';
import ReactFlow, {
  ReactFlowProvider,
  Background,
  Controls,
  MiniMap,
  addEdge,
  useNodesState,
  useEdgesState
} from 'reactflow';
import 'reactflow/dist/style.css';
import api from '../services/api';
import { toast } from 'react-toastify';

// Custom node components
import NodeTypeSelector from '../components/workflow/NodeTypeSelector';
import NodeConfigPanel from '../components/workflow/NodeConfigPanel';
import CustomNode from '../components/workflow/CustomNode';

// New components
import Modal from '../components/common/Modal';
import WorkflowEditorTabs from '../components/workflow/WorkflowEditorTabs';
import WorkflowEditorActions from '../components/workflow/WorkflowEditorActions';
import NodeTester from '../components/workflow/NodeTester';
import ExecutionResults from '../components/execution/ExecutionResults';

const WorkflowEditor = () => {
  const { id } = useParams();
  const navigate = useNavigate();
  const reactFlowWrapper = useRef(null);
  const [reactFlowInstance, setReactFlowInstance] = useState(null);
  const [nodes, setNodes, onNodesChange] = useNodesState([]);
  const [edges, setEdges, onEdgesChange] = useEdgesState([]);
  const [workflow, setWorkflow] = useState({
    name: 'New Workflow',
    description: ''
  });
  const [nodeTypes, setNodeTypes] = useState([]);
  const [selectedNode, setSelectedNode] = useState(null);
  const [isAddingNode, setIsAddingNode] = useState(false);
  const [isSaving, setIsSaving] = useState(false);
  const [isLoading, setIsLoading] = useState(true);
  
  // New state variables
  const [showNodeTester, setShowNodeTester] = useState(false);
  const [showExecutionResults, setShowExecutionResults] = useState(false);
  const [latestExecutionId, setLatestExecutionId] = useState(null);
  const [currentVersion, setCurrentVersion] = useState(1);
  const [showTabs, setShowTabs] = useState(true);

  // Node type definitions for React Flow
  const nodeTypeDefinitions = {
    customNode: CustomNode
  };

  // Load workflow if editing existing one
  useEffect(() => {
    const fetchData = async () => {
      try {
        setIsLoading(true);
        
        // Fetch available node types
        const nodeTypesResponse = await api.get('/api/nodes');
        setNodeTypes(nodeTypesResponse.data.nodes || []);
        
        // If editing existing workflow, fetch it
        if (id && id !== 'new') {
          const workflowResponse = await api.get(`/api/workflows/${id}`);
          const workflowData = workflowResponse.data.workflow;
          
          setWorkflow({
            id: workflowData.id,
            name: workflowData.name,
            description: workflowData.description
          });
          
          // Set current version
          setCurrentVersion(workflowData.version || 1);
          
          // Convert backend nodes/connections to React Flow format
          const flowNodes = workflowData.nodes.map(node => ({
            id: node.id,
            type: 'customNode',
            position: { x: node.position_x, y: node.position_y },
            data: { 
              label: node.name,
              nodeType: node.type,
              config: node.config || {}
            }
          }));
          
          const flowEdges = workflowData.connections.map(conn => ({
            id: conn.id,
            source: conn.source_node_id,
            target: conn.target_node_id,
            sourceHandle: conn.source_handle,
            targetHandle: conn.target_handle
          }));
          
          setNodes(flowNodes);
          setEdges(flowEdges);
        }
      } catch (error) {
        console.error('Error loading workflow editor:', error);
        toast.error('Failed to load workflow editor');
      } finally {
        setIsLoading(false);
      }
    };
    
    fetchData();
  }, [id]);

  // Handle connections between nodes
  const onConnect = useCallback((params) => {
    setEdges((eds) => addEdge(params, eds));
  }, [setEdges]);

  // Handle drag over for node palette
  const onDragOver = useCallback((event) => {
    event.preventDefault();
    event.dataTransfer.dropEffect = 'move';
  }, []);

  // Handle drop from node palette
  const onDrop = useCallback(
    (event) => {
      event.preventDefault();

      const reactFlowBounds = reactFlowWrapper.current.getBoundingClientRect();
      const nodeType = event.dataTransfer.getData('application/reactflow/type');
      const nodeData = JSON.parse(event.dataTransfer.getData('application/reactflow/data'));

      // Check if the dropped element is valid
      if (typeof nodeType === 'undefined' || !nodeType) {
        return;
      }

      const position = reactFlowInstance.project({
        x: event.clientX - reactFlowBounds.left,
        y: event.clientY - reactFlowBounds.top,
      });
      
      const newNode = {
        id: `node_${Date.now()}`,
        type: 'customNode',
        position,
        data: { 
          label: nodeData.name,
          nodeType: nodeData.type,
          config: {}
        },
      };

      setNodes((nds) => nds.concat(newNode));
      setSelectedNode(newNode);
    },
    [reactFlowInstance, setNodes]
  );

  // Handle node selection
  const onNodeClick = useCallback((event, node) => {
    setSelectedNode(node);
  }, []);

  // Handle node config update
  const onNodeConfigUpdate = useCallback((nodeId, config) => {
    setNodes((nds) =>
      nds.map((node) => {
        if (node.id === nodeId) {
          return {
            ...node,
            data: {
              ...node.data,
              config
            }
          };
        }
        return node;
      })
    );
  }, [setNodes]);

  // Handle pane click (deselect node)
  const onPaneClick = useCallback(() => {
    setSelectedNode(null);
  }, []);

  // Handle workflow name change
  const handleNameChange = (e) => {
    setWorkflow({
      ...workflow,
      name: e.target.value
    });
  };

  // Handle workflow description change
  const handleDescriptionChange = (e) => {
    setWorkflow({
      ...workflow,
      description: e.target.value
    });
  };

  // Save workflow
  const handleSave = async () => {
    if (!workflow.name.trim()) {
      toast.error('Please enter a workflow name');
      return;
    }
    
    setIsSaving(true);
    
    try {
      // Convert React Flow nodes/edges to backend format
      const workflowNodes = nodes.map(node => ({
        id: node.id,
        name: node.data.label,
        type: node.data.nodeType,
        position_x: node.position.x,
        position_y: node.position.y,
        config: node.data.config
      }));
      
      const workflowConnections = edges.map(edge => ({
        id: edge.id,
        source_node_id: edge.source,
        target_node_id: edge.target,
        source_handle: edge.sourceHandle || 'default',
        target_handle: edge.targetHandle || 'default'
      }));
      
      const workflowData = {
        name: workflow.name,
        description: workflow.description,
        nodes: workflowNodes,
        connections: workflowConnections
      };
      
      let response;
      
      if (id && id !== 'new') {
        // Update existing workflow
        response = await api.put(`/api/workflows/${id}`, workflowData);
        toast.success('Workflow updated successfully');
      } else {
        // Create new workflow
        response = await api.post('/api/workflows', workflowData);
        toast.success('Workflow created successfully');
        navigate(`/workflows/${response.data.workflow.id}`);
      }
    } catch (error) {
      console.error('Error saving workflow:', error);
      toast.error('Failed to save workflow');
    } finally {
      setIsSaving(false);
    }
  };

  // Execute workflow
  const handleExecute = async () => {
    if (!id || id === 'new') {
      toast.error('Please save the workflow before executing');
      return;
    }
    
    try {
      const response = await api.post(`/api/workflows/${id}/execute`);
      toast.success('Workflow execution started');
      handleExecuteWorkflow(response.data.executionId);
    } catch (error) {
      console.error('Error executing workflow:', error);
      toast.error('Failed to execute workflow');
    }
  };
  
  // Duplicate a node
  const handleDuplicateNode = (node) => {
    const newNode = {
      ...node,
      id: `${node.id}-copy-${Date.now()}`,
      position: {
        x: node.position.x + 50,
        y: node.position.y + 50
      }
    };
    
    setNodes((nds) => nds.concat(newNode));
  };

  // Delete a node
  const handleDeleteNode = (node) => {
    setNodes((nds) => nds.filter((n) => n.id !== node.id));
    setEdges((eds) => eds.filter((e) => e.source !== node.id && e.target !== node.id));
    setSelectedNode(null);
  };

  // Handle workflow execution
  const handleExecuteWorkflow = (executionId) => {
    setLatestExecutionId(executionId);
    setShowExecutionResults(true);
  };

  // Handle version restoration
  const handleRestoreVersion = async (version) => {
    try {
      setIsLoading(true);
      const response = await api.get(`/api/workflows/${id}/versions/${version}`);
      const workflowData = response.data.workflow;
      
      // Update workflow data
      setWorkflow({
        ...workflow,
        name: workflowData.name,
        description: workflowData.description
      });
      
      // Convert backend nodes/connections to React Flow format
      const flowNodes = workflowData.nodes.map(node => ({
        id: node.id,
        type: 'customNode',
        position: { x: node.position_x, y: node.position_y },
        data: { 
          label: node.name,
          nodeType: node.type,
          config: node.config || {}
        }
      }));
      
      const flowEdges = workflowData.connections.map(conn => ({
        id: conn.id,
        source: conn.source_node_id,
        target: conn.target_node_id,
        sourceHandle: conn.source_handle,
        targetHandle: conn.target_handle
      }));
      
      setNodes(flowNodes);
      setEdges(flowEdges);
      setCurrentVersion(version);
      toast.success(`Restored workflow to version ${version}`);
    } catch (error) {
      console.error('Error restoring version:', error);
      toast.error('Failed to restore workflow version');
    } finally {
      setIsLoading(false);
    }
  };

  return (
    <div className="h-[calc(100vh-6rem)]">
      {isLoading ? (
        <div className="h-full flex items-center justify-center">
          <div className="animate-spin rounded-full h-12 w-12 border-t-2 border-b-2 border-primary-500"></div>
        </div>
      ) : (
        <div className="h-full flex flex-col">
          {/* Workflow Header */}
          <div className="bg-white p-4 border-b border-gray-200 flex items-center justify-between">
            <div className="flex-1">
              <input
                type="text"
                value={workflow.name}
                onChange={handleNameChange}
                placeholder="Workflow Name"
                className="text-xl font-semibold border-none focus:ring-0 w-full"
              />
              <input
                type="text"
                value={workflow.description}
                onChange={handleDescriptionChange}
                placeholder="Add a description..."
                className="text-sm text-gray-500 border-none focus:ring-0 w-full"
              />
            </div>
            <div className="flex space-x-2 items-center">
              <button
                onClick={handleSave}
                disabled={isSaving}
                className="px-4 py-2 bg-primary-600 text-white rounded-md hover:bg-primary-700 focus:outline-none focus:ring-2 focus:ring-primary-500 focus:ring-offset-2 disabled:opacity-50"
              >
                {isSaving ? 'Saving...' : 'Save'}
              </button>
              
              <WorkflowEditorActions
                workflowId={id}
                selectedNode={selectedNode}
                onDuplicateNode={handleDuplicateNode}
                onDeleteNode={handleDeleteNode}
                onExecuteWorkflow={handleExecuteWorkflow}
                onTestNode={() => setShowNodeTester(true)}
              />
              
              <button
                onClick={() => setShowTabs(!showTabs)}
                className="ml-2 inline-flex items-center p-1 border border-gray-300 rounded text-gray-500 hover:bg-gray-50 focus:outline-none focus:ring-2 focus:ring-offset-2 focus:ring-primary-500"
              >
                {showTabs ? (
                  <svg xmlns="http://www.w3.org/2000/svg" className="h-5 w-5" viewBox="0 0 20 20" fill="currentColor">
                    <path fillRule="evenodd" d="M5 10a1 1 0 011-1h8a1 1 0 110 2H6a1 1 0 01-1-1z" clipRule="evenodd" />
                  </svg>
                ) : (
                  <svg xmlns="http://www.w3.org/2000/svg" className="h-5 w-5" viewBox="0 0 20 20" fill="currentColor">
                    <path fillRule="evenodd" d="M10 5a1 1 0 011 1v3h3a1 1 0 110 2h-3v3a1 1 0 11-2 0v-3H6a1 1 0 110-2h3V6a1 1 0 011-1z" clipRule="evenodd" />
                  </svg>
                )}
              </button>
            </div>
          </div>
          
          {/* Workflow Editor */}
          <div className="flex-1 flex">
            {/* Node Palette */}
            <div className="w-64 bg-white border-r border-gray-200 overflow-y-auto">
              <NodeTypeSelector nodeTypes={nodeTypes} />
            </div>
            
            {/* Flow Canvas */}
            <div className="flex-1 h-full" ref={reactFlowWrapper}>
              <ReactFlowProvider>
                <ReactFlow
                  nodes={nodes}
                  edges={edges}
                  onNodesChange={onNodesChange}
                  onEdgesChange={onEdgesChange}
                  onConnect={onConnect}
                  onInit={setReactFlowInstance}
                  onDrop={onDrop}
                  onDragOver={onDragOver}
                  onNodeClick={onNodeClick}
                  onPaneClick={onPaneClick}
                  nodeTypes={nodeTypeDefinitions}
                  fitView
                >
                  <Controls />
                  <MiniMap />
                  <Background color="#aaa" gap={16} />
                </ReactFlow>
              </ReactFlowProvider>
            </div>
            
            {/* Node Configuration Panel */}
            {selectedNode && (
              <div className="w-80 bg-white border-l border-gray-200 overflow-y-auto">
                <NodeConfigPanel 
                  node={selectedNode} 
                  onConfigUpdate={onNodeConfigUpdate}
                  onClose={() => setSelectedNode(null)}
                />
              </div>
            )}
          </div>
          
          {/* Workflow Tabs */}
          {id && id !== 'new' && showTabs && (
            <div className="border-t border-gray-200">
              <WorkflowEditorTabs
                workflowId={id}
                nodes={nodes}
                currentVersion={currentVersion}
                onRestoreVersion={handleRestoreVersion}
              />
            </div>
          )}
          
          {/* Modals */}
          <Modal
            isOpen={showNodeTester}
            onClose={() => setShowNodeTester(false)}
            title="Test Node"
            size="lg"
          >
            {selectedNode && (
              <NodeTester
                node={selectedNode}
                onClose={() => setShowNodeTester(false)}
              />
            )}
          </Modal>
          
          <Modal
            isOpen={showExecutionResults}
            onClose={() => setShowExecutionResults(false)}
            title="Execution Results"
            size="xl"
          >
            {latestExecutionId && (
              <ExecutionResults
                key={latestExecutionId}
                workflowId={id}
                executionId={latestExecutionId}
                onClose={() => setShowExecutionResults(false)}
              />
            )}
          </Modal>
        </div>
      )}
    </div>
  );
};

export default WorkflowEditor;
